import { State, StateConfigInterface, StateIngestConfigInterface } from '../state';
import { Observer } from '../runtime';
import { Agile } from '../agile';
import { Collection } from '../collection';
export declare class Computed<ComputedValueType = any> extends State<ComputedValueType> {
    config: ComputedConfigInterface;
    computeFunction: ComputeFunctionType<ComputedValueType>;
    deps: Set<Observer>;
    hardCodedDeps: Array<Observer>;
    isComputed: boolean;
    /**
     * A Computed is an extension of the State Class
     * that computes its value based on a specified compute function.
     *
     * The computed value will be cached to avoid unnecessary recomputes
     * and is only recomputed when one of its direct dependencies changes.
     *
     * Direct dependencies can be States and Collections.
     * So when, for example, a dependent State value changes, the computed value is recomputed.
     *
     * [Learn more..](https://agile-ts.org/docs/core/computed/)
     *
     * @public
     * @param agileInstance - Instance of Agile the Computed belongs to.
     * @param computeFunction - Function to compute the computed value.
     * @param config - Configuration object
     */
    constructor(agileInstance: Agile, computeFunction: ComputeFunctionType<ComputedValueType>, config?: CreateComputedConfigInterface);
    /**
     * Forces a recomputation of the cached value with the compute function.
     *
     * [Learn more..](https://agile-ts.org/docs/core/computed/methods/#recompute)
     *
     * @public
     * @param config - Configuration object
     */
    recompute(config?: RecomputeConfigInterface): this;
    /**
     * Assigns a new function to the Computed Class for computing its value.
     *
     * The dependencies of the new compute function are automatically detected
     * and accordingly updated.
     *
     * An initial computation is performed with the new function
     * to change the obsolete cached value.
     *
     * [Learn more..](https://agile-ts.org/docs/core/computed/methods/#updatecomputefunction)
     *
     * @public
     * @param computeFunction - New function to compute the value of the Computed Class.
     * @param deps - Hard coded dependencies on which the Computed Class depends.
     * @param config - Configuration object
     */
    updateComputeFunction(computeFunction: () => ComputedValueType, deps?: Array<DependableAgileInstancesType>, config?: RecomputeConfigInterface): this;
    /**
     * Computes and returns the new value of the Computed Class
     * and autodetects used dependencies in the compute function.
     *
     * @internal
     * @param config - Configuration object
     */
    compute(config?: ComputeConfigInterface): Promise<ComputedValueType>;
    /**
     * Not usable in Computed Class.
     */
    persist(): this;
}
export declare type ComputeFunctionType<ComputedValueType = any> = () => ComputedValueType | Promise<ComputedValueType>;
export interface CreateComputedConfigInterface extends StateConfigInterface {
    /**
     * Hard-coded dependencies the Computed Class should depend on.
     * @default []
     */
    computedDeps?: Array<DependableAgileInstancesType>;
    /**
     * Whether the Computed should automatically detect
     * used dependencies in the specified compute method.
     *
     * Note that the automatic dependency detection does not work
     * in an asynchronous compute method!
     *
     * @default true if the compute method isn't asynchronous, otherwise false
     */
    autodetect?: boolean;
}
export interface ComputedConfigInterface {
    /**
     * Whether the Computed can automatically detect
     * used dependencies in the compute method.
     *
     * Note that the automatic dependency detection does not work
     * in an asynchronous compute method!
     *
     * @default true if the compute method isn't asynchronous, otherwise false
     */
    autodetect: boolean;
}
export interface ComputeConfigInterface {
    /**
     * Whether the Computed can automatically detect
     * used dependencies in the compute method.
     *
     * Note that the automatic dependency detection does not work
     * in an asynchronous compute method!
     *
     * @default true
     */
    autodetect?: boolean;
}
export interface RecomputeConfigInterface extends StateIngestConfigInterface, ComputeConfigInterface {
}
export declare type DependableAgileInstancesType = State | Collection<any> | Observer;
