import { defineConfig, notEqual } from '@agile-ts/utils';
import { LogCodeManager } from '../logCodeManager.js';
import { CallbackSubscriptionContainer } from './subscription/container/CallbackSubscriptionContainer.js';
import { ComponentSubscriptionContainer } from './subscription/container/ComponentSubscriptionContainer.js';

class Runtime {
  constructor(agileInstance) {
    this.currentJob = null;
    this.jobQueue = [];
    this.jobsToRerender = [];
    this.notReadyJobsToRerender = new Set();
    this.isPerformingJobs = false;
    this.bucketTimeout = null;
    this.agileInstance = () => agileInstance;
  }
  ingest(job, config = {}) {
    config = defineConfig(config, {
      perform: !this.isPerformingJobs
    });
    this.jobQueue.push(job);
    LogCodeManager.logIfTags(["runtime"], "16:01:00", [job._key], job);
    if (config.perform) {
      const performJob = this.jobQueue.shift();
      if (performJob)
        this.perform(performJob);
    }
  }
  perform(job) {
    this.isPerformingJobs = true;
    this.currentJob = job;
    job.observer.perform(job);
    job.performed = true;
    job.observer.dependents.forEach((observer) => observer.ingest());
    if (job.rerender)
      this.jobsToRerender.push(job);
    this.currentJob = null;
    LogCodeManager.logIfTags(["runtime"], "16:01:01", [job._key], job);
    if (this.jobQueue.length > 0) {
      const performJob = this.jobQueue.shift();
      if (performJob)
        this.perform(performJob);
    } else {
      this.isPerformingJobs = false;
      if (this.jobsToRerender.length > 0) {
        if (this.agileInstance().config.bucket) {
          if (this.bucketTimeout == null) {
            this.bucketTimeout = setTimeout(() => {
              this.bucketTimeout = null;
              this.updateSubscribers();
            });
          }
        } else
          this.updateSubscribers();
      }
    }
  }
  updateSubscribers() {
    const jobsToRerender = this.jobsToRerender.concat(Array.from(this.notReadyJobsToRerender));
    this.notReadyJobsToRerender = new Set();
    this.jobsToRerender = [];
    if (!this.agileInstance().hasIntegration() || jobsToRerender.length <= 0)
      return false;
    const subscriptionContainerToUpdate = this.extractToUpdateSubscriptionContainer(jobsToRerender);
    if (subscriptionContainerToUpdate.length <= 0)
      return false;
    this.updateSubscriptionContainer(subscriptionContainerToUpdate);
    return true;
  }
  extractToUpdateSubscriptionContainer(jobs) {
    const subscriptionsToUpdate = new Set();
    for (let i = 0; i < jobs.length; i++) {
      const job = jobs[i];
      job.subscriptionContainersToUpdate.forEach((subscriptionContainer) => {
        let updateSubscriptionContainer = true;
        if (!subscriptionContainer.ready) {
          if (!job.config.maxTriesToUpdate || job.timesTriedToUpdateCount < job.config.maxTriesToUpdate) {
            job.timesTriedToUpdateCount++;
            this.notReadyJobsToRerender.add(job);
            LogCodeManager.log("16:02:00", [subscriptionContainer.key], subscriptionContainer);
          } else {
            LogCodeManager.log("16:02:01", [job.config.maxTriesToUpdate], subscriptionContainer);
          }
          return;
        }
        updateSubscriptionContainer = updateSubscriptionContainer && this.handleSelectors(subscriptionContainer, job);
        if (updateSubscriptionContainer) {
          subscriptionContainer.updatedSubscribers.add(job.observer);
          subscriptionsToUpdate.add(subscriptionContainer);
        }
        job.subscriptionContainersToUpdate.delete(subscriptionContainer);
      });
    }
    return Array.from(subscriptionsToUpdate);
  }
  updateSubscriptionContainer(subscriptionsToUpdate) {
    for (let i = 0; i < subscriptionsToUpdate.length; i++) {
      const subscriptionContainer = subscriptionsToUpdate[i];
      if (subscriptionContainer instanceof CallbackSubscriptionContainer)
        subscriptionContainer.callback();
      if (subscriptionContainer instanceof ComponentSubscriptionContainer)
        this.agileInstance().integrations.update(subscriptionContainer.component, this.getUpdatedObserverValues(subscriptionContainer));
      subscriptionContainer.updatedSubscribers.clear();
    }
    LogCodeManager.logIfTags(["runtime"], "16:01:02", [], subscriptionsToUpdate);
  }
  getUpdatedObserverValues(subscriptionContainer) {
    var _a;
    const props = {};
    for (const observer of subscriptionContainer.updatedSubscribers) {
      const key = (_a = subscriptionContainer.subscriberKeysWeakMap.get(observer)) != null ? _a : observer.key;
      if (key != null)
        props[key] = observer.value;
    }
    return props;
  }
  handleSelectors(subscriptionContainer, job) {
    var _a;
    const selectorMethods = (_a = subscriptionContainer.selectorsWeakMap.get(job.observer)) == null ? void 0 : _a.methods;
    if (selectorMethods == null)
      return true;
    const previousValue = job.observer.previousValue;
    const newValue = job.observer.value;
    for (const selectorMethod of selectorMethods) {
      if (notEqual(selectorMethod(newValue), selectorMethod(previousValue)))
        return true;
    }
    return false;
  }
}

export { Runtime };
